(function() {
  var Log, async, autoSpec, autospec, checkPaths, coffee, colorize, config, displayNotification, exec, expresso, fs, gatherFiles, growl, handleError, intialTestComplete, libDir, log, main, path, prodCoffeeFiles, prodCoffeeOpts, prodDeployDir, prodSrcDir, rootDir, runTests, spec, specCoffeeFiles, specCoffeeOpts, specFileDir, stalker, test, unWatchCoffeeFile, util, walk, watchCoffeeFile, writeSpecFile, _ref, _ref2, _ref3;
  fs = require('fs');
  exec = require('child_process').exec;
  util = require('util');
  stalker = require('stalker');
  growl = require('growl');
  path = require('path');
  async = require('async');
  Log = require('coloured-log');
  log = new Log();
  libDir = path.join(path.dirname(fs.realpathSync(__filename)), '../lib');
  rootDir = process.cwd();
  /*
  config = try require "#{rootDir}/dependency/kitkat_config.js" # Provides overrides for paths used.
  config = config?={} # if no config file has been defined this will make it into an empty object and use default values in this sourcecode
  
  prodSrcDir		= config.prodSrcDir?="#{rootDir}/src"		# assumed that code files could be within the project anywhere
  specFileDir		= config.specFileDir?="#{rootDir}/spec"
  prodDeployDir	= config.prodDeployDir?="#{rootDir}/lib"	# This is the location for compiled code so that tests may be run against them, each file is compliled here as well as a combine js
  commonJsMock	= config.commonJsMock?="/mocks/commonJs.js"	# This file will contain any generic mock ups required by tests, individual mock ups needed in a single spec will be done in the spec file itself 
  */
  config = (function() {
    try {
      return require("" + rootDir + "/kitkat.config.js");
    } catch (_e) {}
  })();
  config = config != null ? config : config = {
    noConfig: true,
    noCompile: false,
    safetyOff: false
  };
  prodDeployDir = path.join("" + rootDir, (_ref = config.prodDeployDir) != null ? _ref : config.prodDeployDir = "lib");
  prodSrcDir = path.join("" + rootDir, (_ref2 = config.prodSrcDir) != null ? _ref2 : config.prodSrcDir = "src");
  specFileDir = path.join("" + rootDir, (_ref3 = config.specFileDir) != null ? _ref3 : config.specFileDir = "spec");
  expresso = path.join(path.dirname(fs.realpathSync(__filename)), '../bin/expresso spec/*.coffee -I lib');
  prodCoffeeFiles = [];
  specCoffeeFiles = [];
  intialTestComplete = false;
  autoSpec = false;
  prodCoffeeOpts = "--output " + prodDeployDir;
  specCoffeeOpts = "--output " + specFileDir;
  checkPaths = function(callback) {
    if (config.safetyOff) {
      return callback(null);
    } else {
      return path.exists(prodDeployDir, function(exists) {
        if (!exists) {
          log.warning("The lib path was not found, this has been created.");
          exec("mkdir " + prodDeployDir, function(err, stdout, stderr) {
            if (err) {
              return callback(err);
            }
          });
        }
        return path.exists(specFileDir, function(exists) {
          if (!exists) {
            log.warning("The spec path was not found, this has been created.");
            exec("mkdir " + specFileDir, function(err, stdout, stderr) {
              if (err) {
                return callback(err);
              }
            });
          }
          return path.exists(prodSrcDir, function(exists) {
            if (!exists) {
              return exec("mkdir " + prodSrcDir, function(err, stdout, stderr) {
                if (err) {
                  return callback(err);
                } else {
                  log.warning("The src path was not found, this has been created and a default app.coffee file created.");
                  return exec("touch " + prodSrcDir + "/app.coffee", function(err, stdout, stderr) {
                    log.notice("Compiling lib/app.js from default app.coffee");
                    return exec("coffee --output " + prodDeployDir + " --compile " + prodSrcDir + "/app.coffee", function(err, stdout, stderr) {
                      return callback(err);
                    });
                  });
                }
              });
            } else {
              return callback(null);
            }
          });
        });
      });
    }
  };
  exports.run = function() {
    var args, commands, key, optimist, value;
    log.notice(colorize("[purple]{Kitkat - Kontinuous Integrated Testing Koffee Application Template}"));
    log.notice(colorize("[red]{press CTRL-C to quit}\n\n"));
    if (config.noCompile === true) {
      log.notice(colorize("[bold]{noCompile:} True"));
    }
    if (config.safetyOff === true) {
      log.notice(colorize("[bold]{safetyOff:} True"));
    }
    log.notice(colorize("[bold]{source files:} " + prodSrcDir));
    log.notice(colorize("[bold]{spec files:} " + specFileDir + "\n"));
    optimist = require('optimist');
    args = optimist.argv;
    optimist.alias('test', 't');
    optimist.alias('spec', 's');
    optimist.alias('autospec', 'a');
    optimist.usage(colorize("[purple]{Kitkat - Kontinuous Integrated Testing Koffee Application Template}\n\n [bold]{Default usage} - with no arguments, it will monitor the current project for changes and\n\t\t run all spec files found against all coffee files found."), {
      'test': {
        description: colorize("[green]{runs all tests against your application files in the spec path\n}"),
        required: false
      },
      'spec': {
        description: colorize("[green]{generates spec files based on the supplied path/file, if none\n\t\t  provided then processes all coffee files in project\n}"),
        required: false
      },
      'autospec': {
        description: colorize("[green]{generates new spec files and monitors all files for changes,\n\t\t  automatically runs tests on startup and as files change\n}"),
        required: false
      }
    });
    commands = [];
    for (key in args) {
      value = args[key];
      try {
        switch (key.toLowerCase()) {
          case "t":
          case "test":
          case "s":
          case "spec":
          case "a":
          case "autotest":
            commands.push(key);
            break;
          default:
            if (key !== "$0" && key !== "_") {
              throw "Invaid argument";
            }
        }
      } catch (ex) {
        log.error("" + ex + "\n");
        optimist.showHelp();
        return;
      }
    }
    return checkPaths(function(err) {
      var command, _ref4, _ref5;
      if (err) {
        handleError(err);
        callback(err);
      }
      command = (_ref4 = commands[0]) != null ? _ref4.toLowerCase() : void 0;
      switch (command) {
        case "t":
        case "test":
          log.notice("Initiating tests");
          return test();
        case "s":
        case "spec":
          log.notice("Generating spec files");
          return spec((_ref5 = args.s) != null ? _ref5 : args.s = args.spec);
        case "a":
        case "autospec":
          log.notice("Autospec and testing current project");
          return autospec();
        default:
          log.notice("Monitoring and testing current project");
          return main();
      }
    });
  };
  autospec = function() {
    autoSpec = true;
    return main();
  };
  coffee = function(options, file) {
    if (options == null) {
      options = "";
    }
    log.info("Compiling " + file);
    return exec("coffee " + options + " --compile " + file, function(err, stdout, stderr) {
      if (err) {
        return handleError(err);
      }
    });
  };
  colorize = function(str) {
    var colors;
    colors = {
      bold: 1,
      red: 31,
      green: 32,
      yellow: 33,
      purple: 35,
      cyan: 36
    };
    return str.replace(/\[(\w+)\]\{([^]*?)\}/g, function(_, color, str) {
      return "\u001b[" + colors[color] + "m" + str + "\u001b[0m";
    });
  };
  displayNotification = function(message) {
    var options;
    if (message == null) {
      message = '';
    }
    options = {
      title: 'CoffeeScript',
      image: 'lib/CoffeeScript.png'
    };
    return growl.notify(message, options);
  };
  gatherFiles = function(filepath, extRegex, callback) {
    var file, filesFound;
    filesFound = [];
    file = path.normalize(filepath);
    return fs.stat(file, function(err, stats) {
      if (err) {
        file += extRegex.toString().replace(/\//g, "").replace(/\\/g, "");
        return fs.stat(file, function(err, stats) {
          if (err) {
            callback(err, null);
          }
          if (!err) {
            filesFound.push(file);
            return callback(null, filesFound);
          }
        });
      } else {
        if (stats.isDirectory()) {
          return walk(path.normalize(file + '/'), extRegex, function(err, results) {
            if (err) {
              callback(err, null);
            }
            if (!err) {
              filesFound = results;
              return callback(null, filesFound);
            }
          });
        } else {
          filesFound.push(file);
          return callback(null, filesFound);
        }
      }
    });
  };
  handleError = function(error) {
    log.error(error);
    return displayNotification(error);
  };
  main = function() {
    stalker.watch("" + prodSrcDir, watchCoffeeFile, unWatchCoffeeFile);
    stalker.watch("" + specFileDir, watchCoffeeFile, unWatchCoffeeFile);
    return test();
  };
  runTests = function(callback) {
    var message;
    message = "";
    return exec(expresso, function(err, stdout, stderr) {
      var options, output, _ref4, _ref5;
      if (err) {
        message = "" + ((_ref4 = stderr.match(/Failures.*/)) != null ? _ref4.toString() : void 0);
        message = message.replace(/\[\d*m/g, "");
        output = stderr.toString();
        output = output.replace(/at .*\n/g, "");
        log.error("Test Results\n\n" + output);
        options = {
          title: 'Automated Testing',
          image: "" + libDir + "/error.png"
        };
      } else {
        message = "100% - " + ((_ref5 = stderr.match(/\d* tests/)) != null ? _ref5.toString() : void 0) + " passed";
        options = {
          title: 'Automated Testing',
          image: "" + libDir + "/success.png"
        };
        log.info(colorize("[green]{" + message + "}"));
      }
      growl.notify(message, options);
      return callback(null);
    });
  };
  spec = function(filepath) {
    return async.series({
      gather_files: function(callback) {
        if (filepath === true) {
          filepath = "/";
        }
        filepath = path.join(prodSrcDir, path.normalize(filepath));
        return gatherFiles(filepath, /\.coffee/, function(err, results) {
          prodCoffeeFiles = results;
          return callback(err, results);
        });
      },
      create_specs: function(callback) {
        var file, index, pending, _len, _results;
        pending = prodCoffeeFiles.length;
        _results = [];
        for (index = 0, _len = prodCoffeeFiles.length; index < _len; index++) {
          file = prodCoffeeFiles[index];
          _results.push((function(file, index) {
            var specFile;
            specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee");
            fs.stat(specFile, function(err, stats) {
              var prodFile;
              if (err) {
                prodFile = path.join(prodDeployDir, path.basename(file, '.coffee'));
                return writeSpecFile(specFile, prodFile, function(error, results) {
                  if (error) {
                    return callback(err, null);
                  }
                });
              }
            });
            if (!--pending) {
              return callback(null, null);
            }
          })(file, index));
        }
        return _results;
      }
    }, function(err, results) {
      if (err) {
        return handleError(err);
      }
    });
  };
  test = function() {
    return async.series({
      gather_files: function(callback) {
        return async.parallel({
          gather_specs: function(callback) {
            return gatherFiles(specFileDir, /\.spec\.coffee/, function(err, results) {
              specCoffeeFiles = results;
              return callback(err, results);
            });
          },
          gather_prod: function(callback) {
            return gatherFiles(prodSrcDir, /\.coffee/, function(err, results) {
              prodCoffeeFiles = results;
              return callback(err, results);
            });
          }
        }, function(err, results) {
          return callback(err, results);
        });
      },
      compile_prod: function(callback) {
        var file, index, pending, _len, _results;
        if (config.noCompile === true) {
          return callback(null, null);
        } else {
          pending = prodCoffeeFiles.length;
          _results = [];
          for (index = 0, _len = prodCoffeeFiles.length; index < _len; index++) {
            file = prodCoffeeFiles[index];
            _results.push((function(file, index) {
              return exec("coffee --output " + prodDeployDir + " --compile " + file, function(err, stdout, stderr) {
                if (!--pending) {
                  return callback(null, null);
                }
              });
            })(file, index));
          }
          return _results;
        }
      },
      create_specs: function(callback) {
        var file, index, pending, _len, _results;
        if (autoSpec === true) {
          pending = prodCoffeeFiles.length;
          _results = [];
          for (index = 0, _len = prodCoffeeFiles.length; index < _len; index++) {
            file = prodCoffeeFiles[index];
            _results.push((function(file, index) {
              var specFile;
              specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee");
              return fs.stat(specFile, function(err, stats) {
                var prodFile;
                if (err) {
                  prodFile = path.join(prodDeployDir, path.basename(file, '.coffee'));
                  writeSpecFile(specFile, prodFile, function(err, results) {
                    if (err) {
                      return callback(err, null);
                    }
                  });
                  specCoffeeFiles.push(specFile);
                  if (!--pending) {
                    return callback(null, null);
                  }
                } else {
                  if (!--pending) {
                    return callback(null, null);
                  }
                }
              });
            })(file, index));
          }
          return _results;
        } else {
          return callback(null, null);
        }
      },
      run_specs: function(callback) {
        return runTests(function(err) {
          intialTestComplete = true;
          return callback(err, null);
        });
      }
    });
  };
  unWatchCoffeeFile = function(err, file) {
    if (("" + file).match(/\.coffee/) != null) {
      util.warning("" + file + " removed.");
      fs.unwatchFile("" + file);
      return runTests(function(err) {});
    }
  };
  walk = function(dir, extRegex, callback) {
    var pending, results;
    results = [];
    pending = 0;
    return fs.readdir(dir, function(err, list) {
      var file, _i, _len, _results;
      if (err) {
        callback(err, null);
      }
      pending = list.length;
      _results = [];
      for (_i = 0, _len = list.length; _i < _len; _i++) {
        file = list[_i];
        _results.push((function(file) {
          file = dir + file;
          return fs.stat(file, function(err, stat) {
            if (stat && stat.isDirectory()) {
              file += "/";
              if (("" + file).match(/node_modules/) != null) {
                if (!--pending) {
                  return callback(null, results);
                }
              } else {
                return walk(file, extRegex, function(err, res) {
                  results = results.concat(res);
                  if (!--pending) {
                    return callback(null, results);
                  }
                });
              }
            } else {
              if (file.match(extRegex)) {
                results.push(file);
              }
              if (!--pending) {
                return callback(null, results);
              }
            }
          });
        })(file));
      }
      return _results;
    });
  };
  watchCoffeeFile = function(err, file) {
    var specFile;
    if (!(("" + file).match(/\.spec\.coffee/) != null) && (("" + file).match(/\.coffee/) != null)) {
      log.notice("Watch added for " + colorize("[cyan]{" + file + "}"));
      fs.unwatchFile("" + file);
      fs.watchFile("" + file, function(curr, prev) {
        if (+curr.mtime !== +prev.mtime) {
          log.notice("Saw change in " + colorize("[cyan]{" + file + "}"));
          if (!config.noCompile === true) {
            return exec("coffee --output " + prodDeployDir + " --compile " + file, function(err, stdout, stderr) {
              return runTests(function(err) {});
            });
          }
        }
      });
      if (autoSpec === true) {
        specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee");
        return fs.stat(specFile, function(err, stats) {
          var prodFile;
          if (err) {
            prodFile = path.join(prodDeployDir, path.basename(file, '.coffee'));
            return writeSpecFile(specFile, prodFile, function(err, results) {
              if (err) {
                callback(err, null);
              }
              fs.unwatchFile("" + specFile);
              fs.watchFile("" + specFile, function(curr, prev) {
                if (+curr.mtime !== +prev.mtime) {
                  log.notice("Saw change in " + colorize("[cyan]{" + specFile + "}"));
                  return runTests(function(err) {});
                }
              });
              return specCoffeeFiles.push(specFile);
            });
          }
        });
      }
    } else {
      if (("" + file).match(/\.spec\.coffee/)) {
        log.notice("Watch added for " + colorize("[cyan]{" + file + "}"));
        fs.unwatchFile("" + file);
        return fs.watchFile("" + file, function(curr, prev) {
          if (+curr.mtime !== +prev.mtime) {
            log.notice("Saw change in " + colorize("[cyan]{" + file + "}"));
            return runTests(function(err) {});
          }
        });
      }
    }
  };
  writeSpecFile = function(specFile, prodFile, callback) {
    var libFile, specContent;
    libFile = path.basename(prodFile, path.extname(prodFile));
    specContent = ["replace with libfile", "assert = require('assert')", "module.exports =", "\t\"You need to modify this with real test cases and thus it fails until this file is modified.\": ->", "\t\tassert.ok false, \"it will fail\""];
    specContent[0] = "" + (libFile.replace(/\-/, "_")) + " = require('" + libFile + "')";
    return fs.writeFile(specFile, specContent.join('\n'), 'utf8', function(err) {
      if (err) {
        callback(err, null);
      }
      log.notice("Created specfile '" + specFile + "'");
      return callback(null, null);
    });
  };
}).call(this);
