fs	   = require 'fs'
{exec} = require 'child_process'
util   = require 'util'
stalker = require 'stalker'
growl = require 'growl'
path = require 'path'
async = require 'async'
Log = require 'coloured-log'

log = new Log()

libDir			= path.join(path.dirname(fs.realpathSync(__filename)), '../lib')
rootDir			= process.cwd()

###
config = try require "#{rootDir}/dependency/kitkat_config.js" # Provides overrides for paths used.
config = config?={} # if no config file has been defined this will make it into an empty object and use default values in this sourcecode

prodSrcDir		= config.prodSrcDir?="#{rootDir}/src"		# assumed that code files could be within the project anywhere
specFileDir		= config.specFileDir?="#{rootDir}/spec"
prodDeployDir	= config.prodDeployDir?="#{rootDir}/lib"	# This is the location for compiled code so that tests may be run against them, each file is compliled here as well as a combine js
commonJsMock	= config.commonJsMock?="/mocks/commonJs.js"	# This file will contain any generic mock ups required by tests, individual mock ups needed in a single spec will be done in the spec file itself 
###

config = try require "#{rootDir}/kitkat.config.js"
config = config?={noConfig: true, noCompile: false, safetyOff: false}


prodDeployDir	= path.join("#{rootDir}", config.prodDeployDir?="lib")
prodSrcDir		= path.join("#{rootDir}", config.prodSrcDir?="src")		# assumed that code files could be within the project anywhere
specFileDir		= path.join("#{rootDir}", config.specFileDir?="spec")

expresso = path.join(path.dirname(fs.realpathSync(__filename)), '../bin/expresso spec/*.coffee -I lib')

prodCoffeeFiles = []
specCoffeeFiles = []
intialTestComplete = false
autoSpec = false

prodCoffeeOpts = "--output #{prodDeployDir}"
specCoffeeOpts = "--output #{specFileDir}"


# Verify paths exist, create if they do not, send back error if any issues occured.
checkPaths = (callback) ->
	if config.safetyOff
		callback null # this will cause us not to check or autogenerate content, depending on whether your test cases require it or not.
	else
		path.exists prodDeployDir, (exists) ->
		
			if not exists
				log.warning "The lib path was not found, this has been created."
				exec "mkdir #{prodDeployDir}", (err, stdout, stderr) ->
					if err
						callback err
			path.exists specFileDir, (exists) ->
				if not exists
					log.warning "The spec path was not found, this has been created."
					exec "mkdir #{specFileDir}", (err, stdout, stderr) ->
						if err
							callback err
				path.exists prodSrcDir, (exists) ->
					if not exists
						exec "mkdir #{prodSrcDir}", (err, stdout, stderr) ->
							if err
								callback err
							else
								log.warning "The src path was not found, this has been created and a default app.coffee file created."
								exec "touch #{prodSrcDir}/app.coffee", (err, stdout, stderr) ->
									log.notice "Compiling lib/app.js from default app.coffee"
									exec "coffee --output #{prodDeployDir} --compile #{prodSrcDir}/app.coffee", (err,stdout, stderr) ->
										callback err
					else
						callback null
	

# Entry point to kitkat
exports.run = ->
	# Commandline processor
	log.notice colorize("[purple]{Kitkat - Kontinuous Integrated Testing Koffee Application Template}")
	log.notice colorize("[red]{press CTRL-C to quit}\n\n")
	if config.noCompile==true
		log.notice colorize("[bold]{noCompile:} True")
	if config.safetyOff==true
		log.notice colorize("[bold]{safetyOff:} True")

	log.notice colorize("[bold]{source files:} #{prodSrcDir}")
	log.notice colorize("[bold]{spec files:} #{specFileDir}\n")

	optimist = require 'optimist'
	args = optimist.argv

	optimist.alias 'test', 't'
	optimist.alias 'spec', 's'
	optimist.alias 'autospec', 'a'
	optimist.usage(colorize("[purple]{Kitkat - Kontinuous Integrated Testing Koffee Application Template}\n\n [bold]{Default usage} - with no arguments, it will monitor the current project for changes and\n\t\t run all spec files found against all coffee files found."), {
		'test': {description: colorize("[green]{runs all tests against your application files in the spec path\n}"), required: false},
		'spec': {description: colorize("[green]{generates spec files based on the supplied path/file, if none\n\t\t  provided then processes all coffee files in project\n}"), required: false},
		'autospec': {description: colorize("[green]{generates new spec files and monitors all files for changes,\n\t\t  automatically runs tests on startup and as files change\n}"), required: false},
		})
	commands = []
	for key, value of args
		try 
			switch key.toLowerCase()
				when "t", "test", "s", "spec", "a", "autotest"
					commands.push key
				else
					if key !="$0" and key !="_"
						throw "Invaid argument"
		catch ex
			log.error "#{ex}\n"
			optimist.showHelp() 
			return
	
	
	checkPaths (err) ->
		if err
			handleError(err)
			callback err

		command = commands[0]?.toLowerCase()
		
		switch command
			when "t", "test"
				log.notice "Initiating tests"
				test() # args.t ?= args.test
			when "s", "spec"
				log.notice "Generating spec files"
				spec args.s ?= args.spec
			when "a", "autospec"
				log.notice "Autospec and testing current project"
				autospec()
			else
				log.notice "Monitoring and testing current project"
				main()

#########################################
# Functions follow in alphabetical order
#########################################

autospec = ->
	autoSpec = true
	main()

coffee = (options = "", file) ->
	log.info "Compiling #{file}"
	exec "coffee #{options} --compile #{file}", (err, stdout, stderr) -> 
		handleError(err) if err

colorize = (str) ->
	colors = 
		bold: 1
		red: 31
		green: 32
		yellow: 33
		purple: 35
		cyan: 36

	str.replace /\[(\w+)\]\{([^]*?)\}/g, (_, color, str) ->
		"\u001b[" + colors[color] + "m" + str + "\u001b[0m"

displayNotification = (message = '') -> 
	options = {
		title: 'CoffeeScript'
		image: 'lib/CoffeeScript.png'
	}
	growl.notify message, options

gatherFiles = (filepath, extRegex, callback) ->
	filesFound = []
	file = path.normalize(filepath)
	fs.stat file, (err,stats) ->
		if err
			file += extRegex.toString().replace(/\//g,"").replace(/\\/g,"")
			fs.stat file, (err,stats) ->
				callback err, null if err 
				if not err
					filesFound.push file 
					callback null, filesFound
		else
			if stats.isDirectory()
				walk path.normalize(file + '/'), extRegex, (err,results) ->
					callback err, null if err
					if not err
						filesFound = results	
						callback null, filesFound
			else
				filesFound.push file
				callback null, filesFound

handleError = (error) -> 
	log.error error
	displayNotification error

main = ->
	stalker.watch "#{prodSrcDir}" , watchCoffeeFile, unWatchCoffeeFile
	stalker.watch "#{specFileDir}" , watchCoffeeFile, unWatchCoffeeFile
	test()

runTests = (callback) ->
	message = ""
	exec expresso, (err, stdout, stderr) ->
		if err
			message = "" + stderr.match(/Failures.*/)?.toString()		# Extract failure count
			message = message.replace(/\[\d*m/g, "") 					# stripping special formatting from output
			output = stderr.toString()
			output = output.replace(/at .*\n/g, "")
			log.error "Test Results\n\n" + output
			options = {
				title: 'Automated Testing'
				image: "#{libDir}/error.png"
			}

		else
			message = "100% - " + stderr.match(/\d* tests/)?.toString() + " passed"		# Extract Test count
			options = {
				title: 'Automated Testing'
				image: "#{libDir}/success.png"
			}
			log.info colorize("[green]{#{message}}")

		growl.notify message, options
		callback null

spec = (filepath) ->
	async.series
		gather_files: (callback) ->
			filepath = "/" if (filepath==true)
			filepath = path.join(prodSrcDir, path.normalize(filepath))
			gatherFiles filepath, /\.coffee/, (err, results) ->
				prodCoffeeFiles = results
				callback err, results
		create_specs: (callback) ->
			pending = prodCoffeeFiles.length
			for file, index in prodCoffeeFiles then do (file, index) ->
				specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee")
				fs.stat specFile, (err,stats) ->
					if err # File does not exist
						prodFile = path.join(prodDeployDir, path.basename(file, '.coffee') )
						writeSpecFile specFile, prodFile, (error, results) ->
							callback err, null if error
				callback null, null unless --pending
	, (err, results) ->
		handleError(err) if err

test = ->
	async.series
		gather_files: (callback) ->
			async.parallel
				gather_specs: (callback) ->
					gatherFiles specFileDir, /\.spec\.coffee/, (err, results) ->
						specCoffeeFiles = results
						callback err, results
				gather_prod: (callback) ->
					gatherFiles prodSrcDir, /\.coffee/, (err, results) ->
						prodCoffeeFiles = results
						callback err, results
			, (err, results) ->
				callback err, results

		compile_prod: (callback) ->
			if config.noCompile==true
				callback null,null # Some projects do not require us to recompile anything, rather just watch the files for changes
			else
				pending = prodCoffeeFiles.length
				for file, index in prodCoffeeFiles then do (file, index) ->
					exec "coffee --output #{prodDeployDir} --compile #{file}", (err,stdout, stderr) ->
						callback null, null unless --pending

		create_specs: (callback) ->
			if autoSpec==true
				pending = prodCoffeeFiles.length
				for file, index in prodCoffeeFiles then do (file, index) ->
					specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee")
					fs.stat specFile, (err,stats) ->
						if err # File does not exist
							prodFile = path.join(prodDeployDir, path.basename(file, '.coffee'))
							writeSpecFile specFile, prodFile, (err, results) ->
								callback err, null if err
							specCoffeeFiles.push specFile
							callback null, null unless --pending
						else
							callback null, null unless --pending
			else 
				callback null, null

		run_specs: (callback) ->
			runTests (err) ->
				intialTestComplete = true
				callback err, null

unWatchCoffeeFile = (err, file) ->
	if "#{file}".match(/\.coffee/)?
		util.warning "#{file} removed."
		fs.unwatchFile "#{file}"
		runTests (err) ->
			return

walk = (dir, extRegex, callback) ->
	results = []
	pending = 0
	fs.readdir dir, (err, list) ->
		callback err, null if err
		pending = list.length
		for file in list then do(file) ->
			file = dir + file
			fs.stat file, (err, stat) ->
				if stat and stat.isDirectory()
					file += "/"
					if "#{file}".match(/node_modules/)?
						callback null,results unless --pending
					else
						walk file, extRegex, (err, res) ->
							results = results.concat(res)
							callback null, results unless --pending
				else
					if file.match(extRegex)
						results.push file
					callback null, results unless --pending

watchCoffeeFile = (err, file) ->
	# Test that we are looking at a .coffee file which is not a spec file
	if not "#{file}".match(/\.spec\.coffee/)? and "#{file}".match(/\.coffee/)?

		log.notice "Watch added for " + colorize("[cyan]{#{file}}")
		fs.unwatchFile "#{file}"
		fs.watchFile "#{file}", (curr, prev) ->
			if +curr.mtime isnt +prev.mtime
				log.notice "Saw change in " + colorize("[cyan]{#{file}}")
				if not config.noCompile==true
					exec "coffee --output #{prodDeployDir} --compile #{file}", (err,stdout, stderr) ->
						runTests (err) ->
							return
		if autoSpec==true
			specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee")
			fs.stat specFile, (err,stats) ->
				if err # File does not exist
					prodFile = path.join(prodDeployDir, path.basename(file, '.coffee'))
					writeSpecFile specFile, prodFile, (err, results) ->
						callback err, null if err
						fs.unwatchFile "#{specFile}"
						fs.watchFile "#{specFile}", (curr, prev) ->
							if +curr.mtime isnt +prev.mtime
								log.notice "Saw change in " + colorize("[cyan]{#{specFile}}")
								runTests (err) ->
									return
						specCoffeeFiles.push specFile
	# Watching spec file
	else
		if "#{file}".match(/\.spec\.coffee/)
			log.notice "Watch added for " + colorize("[cyan]{#{file}}")
			fs.unwatchFile "#{file}"
			fs.watchFile "#{file}", (curr, prev) ->
				if +curr.mtime isnt +prev.mtime
					log.notice "Saw change in " + colorize("[cyan]{#{file}}")
					runTests (err) ->
						return

# Creates the default spec file.
writeSpecFile = (specFile, prodFile, callback) ->
	libFile = path.basename(prodFile, path.extname(prodFile))
	specContent = ["replace with libfile", "assert = require('assert')", "module.exports =", "\t\"You need to modify this with real test cases and thus it fails until this file is modified.\": ->", "\t\tassert.ok false, \"it will fail\"" ]
	specContent[0] = "#{libFile.replace(/\-/,"_")} = require('#{libFile}')"

	fs.writeFile specFile, specContent.join('\n'), 'utf8', (err) ->
		callback err, null if err
		log.notice "Created specfile '#{specFile}'"
		callback null, null